# test (penalized) linear regression with a sine function

# test points where we want to fit
vX <- seq(0,1,0.03) 
vY <- sin(2 * pi * vX) + rnorm(length(vX),0,0.2)

plot(vX,vY,type="p")

# create the feature matrix; we fit with polynomials; k indicates the max order of polynomials we consider
k <- 3 
featureVec <- function(x) x^seq(0,k)
mX <- t(sapply(vX, featureVec))

# solve the linear regression
fitLm <- lm(vY ~ -1 + mX)
beta <- fitLm$coefficients 
beta[is.na(beta)]=0

# plot the regression function
RegFun <- Vectorize(function(x) featureVec(x) %*% beta)
plot(RegFun, 0, 1, add=TRUE)
plot(function(x) sin(2*pi*x), 0, 1, add=TRUE, lty=2)

# now fit the data via lasso regularization
library(glmnet)
fitGlm <- glmnet(mX, vY)
# obtain "best" lambda via cross validation
cv <- cv.glmnet(mX, vY)
predFun <- Vectorize(function(x) {
                     mNewX <- matrix(featureVec(x),1,k+1)
                     predict(fitGlm, mNewX, type="response", s=cv$lambda.min)
})
plot(predFun, 0, 1, add=TRUE, col="red")

# compare coefficients of linear model and lasso regularization for cross validated lambda
print(beta)
nPosOfCvLambda <- match(cv$lambda.min, fitGlm$lambda)
print(fitGlm$beta[,nPosOfCvLambda])

